#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from scet.globales import *

import os
import logging
import time
import sys
import traceback
import time
import subprocess
import pip
from tkinter import Tk, ttk, Toplevel, messagebox
from os.path import join, abspath, dirname

from mysql import connector

from scet.libsa import Ventana, Base_Entrada, Formulario, Bucle, Tabla,\
    Reporte, seguro, ControladorEstado, VentanaError
from scet.bd_xml import Tabla
from scet import Ventanas


log = logging.getLogger(__name__)


class inicio_seccion(Ventana, Base_Entrada):
    archivo_gui = "Inicio_seccion"
    dir = join(CARPETA_TRABAJO, 'BD_login.xml')
    ok = False

    def __init__(self, master):
        self.master = master
        self.master.wm_title(NOMBRE)
        Ventana.__init__(self, self.master, self.archivo_gui)
        self.gui.get_object('contraseña').focus()

        self.bd = Tabla()
        log.debug('Pantalla de incio de sesion creada')

        try:
            log.debug('Buscar archivo informacion comun')
            with open(self.dir, 'r') as preferencias:
                self.bd.xml = preferencias.read()

        except IOError as err:
            log.debug('Archivo no encontrada, creando uno generico')
            self._crear_archivo_preferencias()

            with open(self.dir, 'w') as preferencias:
                preferencias.write(self.bd.xml)

        self._sincronisar('formulario', 0, self.bd, 'none_a_str')
        log.debug('Preferencias escritas en el formulario')

    @seguro
    def aceptar_ser(self, event=None):
        self.bd[0] = self.datos
        self.config = self.bd[0]
        c = connector.connect(**self.config)
        self.ok = True
        self.master.destroy()
        log.debug('Itento de coneccion exitoso')

    @seguro
    def salir(self):
        self.master.destroy()

    @seguro
    def guardar(self):
        self._crear_archivo_preferencias()
        self.bd[0] = self.datos

        with open(self.dir, 'w') as preferencias:
            preferencias.write(self.bd.xml)

        messagebox.showinfo(
            message='Guardado exitosamente',
            icon="info",
            title="guardado exitoso"
            )

    def _crear_archivo_preferencias(self):
        self.bd = Tabla('Preferencias')
        self.bd.agregar_columna(campo='host', tipo='str', defecto='127.0.0.1')
        self.bd.agregar_columna(campo='port', tipo='str', defecto='3306')
        self.bd.agregar_columna(campo='user', tipo='str', defecto='')
        self.bd.agregar_columna(campo='password', tipo='str', defecto='')
        self.bd.agregar_columna(campo='db', tipo='str', defecto='SCET')
        #self.bd.agregar_columna(campo='file', tipo='str', defecto='')
        self.bd += ()


class programa(Ventana):
    formulario = None

    _estilos = {
        'linux':'clam',
        'win32':'clam',
        }

    _fuente = ('Helvetica', 11)
    _fuente_titulo = _fuente + ('bold',)

    def __init__(self, master):
        self.master = master
        self.master.report_callback_exception = self.report_callback_exception
        master.wm_title(NOMBRE)
        Ventana.__init__(self, master, 'Main')
        #~ __estado__ = ControladorEstado(self.gui.get_variable('__estado__'))
        #~ self.__estado__ = logging.StreamHandler(stream=__estado__)
        #~ self.__estado__.terminator = ''
        #~ self.__estado__.setLevel(logging.INFO)
        #~ logging.getLogger('').addHandler(self.__estado__)
        self.marco = self.gui.get_object('__marco__')
        self._estilo()
        self._login()
        self._menu()

    def report_callback_exception(self, *args):
        VentanaError(args)

    @seguro
    def _login(self):
        self.master.withdraw()
        ventana = Toplevel(self.master)
        self.login = inicio_seccion(ventana)
        self.master.wait_window(ventana)
        if not self.login.ok:
            log.debug('Pantalla de inicio de secion cerrada, cerrando programa')
            sys.exit(0)
        else:
            self.con = self.login.config
            self.master.deiconify()

    @seguro
    def _estilo(self):
        log.debug('Creando estilo grafico')
        self.estilo = ttk.Style()

        log.debug('Sistema operativo es {}'.format(sys.platform))
        for os, estilo in self._estilos.items():
            if os in sys.platform:
                log.debug('Seleccionando estilo {}'.format(estilo))
                s = estilo
                break
        else:
            s = 'default'
            log.debug('No se encontro un estilo para el OS, usando default')

        self.estilo.theme_use(s)

        #conf
        self.estilo.configure('.', font=self._fuente, background='white')
        self.estilo.configure('Subtitulo.TLabel', font=self._fuente_titulo)
        self.estilo.configure('TNotebook.Tab', font=self._fuente_titulo)
        self.estilo.configure('TButton', font=self._fuente_titulo)

    def _menu(self):
        menu = self.gui.get_object('__menu_usuario__')

        entradaTaller = menu.insert('', 'end', text='Entrada a Taller', tags=('fila', 'a'))
        controlClientes = menu.insert('', 'end', text='Control de Clientes', tags=('fila', 'b'))
        controlRecursos = menu.insert('', 'end', text='Control de Recursos', tags=('fila', 'a'))
        controlEntradas = menu.insert('', 'end', text='Equipos en Taller', tags=('fila', 'b'))
        historialEntradas = menu.insert('', 'end', text='Historial de Trabajos', tags=('fila', 'a'))

        #~ menu.bind('<Double-Button-1>', self._selecionar_menu)
        #~ menu.bind('<Return>', self._selecionar_menu)
        menu.tag_bind('fila', '<Double-Button-1>', self._selecionar_menu)
        menu.tag_bind('fila', '<Return>', self._selecionar_menu)

        menu.tag_configure('fila', font=self._fuente_titulo)
        menu.tag_configure('a', background='white')
        menu.tag_configure('b', background='gray')

        self._mapeo_menu_formulario = {
            entradaTaller:Ventanas.Entrada_Taller,
            controlClientes:Ventanas.Control_Clientes,
            controlRecursos:Ventanas.Control_Recursos,
            controlEntradas:Ventanas.Control_Entradas,
            historialEntradas:Ventanas.Historial_Entradas,
        }

    def _selecionar_menu(self, evento):
        menu = evento.widget
        if not menu.selection(): return None

        objeto = menu.selection()[0]

        if objeto in self._mapeo_menu_formulario:
            formulario = self._mapeo_menu_formulario[objeto]
        log.debug('menu: {menu}, objeto: {objeto}, formulario: {formulario}'.format(**locals()))

        if self.formulario: self.formulario.destruir()

        self.formulario = formulario(self.marco, self.con)


    @seguro
    def acerca(self):
        acerca()


class acerca(Ventana):

    def __init__(self):
        Ventana.__init__(self, OBTENER_ROOT(), 'Acerca')
        self.valores()

    def valores(self):
        self.gui.get_variable('vercion').set('{} - Ver. {}'.format(NOMBRE,VERSION))
        self.gui.get_variable('titulo').set(TITULO)
        self.gui.get_variable('fecha').set(FECHA.strftime(FORMATO_FECHA))
        self.gui.get_variable('autor').set(AUTOR)
        self.gui.get_variable('contacto').set(CONTACTO)

    def licencia(self):
        archivo = abspath(join(dirname(__file__),'Licencia.html'))
        subprocess.Popen([archivo])

    def manual(self):
        proc = join(CARPETA_MADRE, 'manual.pdf')
        if sys.platform.startswith('linux'):
            subprocess.Popen(['xdg-open', proc])
        if sys.platform.startswith('win'):
            os.startfile(proc)

    def actualisar(self):
        actualisar()

@seguro
def actualisar(self):
    sys.path.insert(0, 'pkgs')
    log.debug('Ejecutando Actualisador')
    if sys.platform.startswith('linux'):
        r = [ 'install', 'scet', '--user', '--upgrade']
    elif sys.platform.startswith('win'):
        r = ['install', 'scet', '--upgrade']
    else:
        r = ['install', 'scet', '--upgrade']
    pip.main(r)
    messagebox.showinfo(
        title='Actualisacion Completa',
        message='El programa se cerrara ahora para aplicar los cambios, {}'.format(r),
        )
    sys.exit(0)

def main():
    root = OBTENER_ROOT()
    w, h = root.winfo_screenwidth(), root.winfo_screenheight()
    root.geometry("%dx%d+0+0" % (w, h))
    scet = programa(root)
    root.mainloop()

if __name__ == '__main__':
    main()
